function varargout = interface(varargin)
global intflag;
global cenx;
global ceny;

cenx = 23;
ceny = 21.7;

gui_Singleton = 1;
gui_State = struct('gui_Name',       mfilename, ...
                   'gui_Singleton',  gui_Singleton, ...
                   'gui_OpeningFcn', @interface_OpeningFcn, ...
                   'gui_OutputFcn',  @interface_OutputFcn, ...
                   'gui_LayoutFcn',  [] , ...
                   'gui_Callback',   []);
if nargin && ischar(varargin{1})
    gui_State.gui_Callback = str2func(varargin{1});
end

if nargout
    [varargout{1:nargout}] = gui_mainfcn(gui_State, varargin{:});
else
    gui_mainfcn(gui_State, varargin{:});
end

intflag = false;

%%% SERIAL COMMUNICATION %%%

function writeserial(msg)
global s;
set(s,'Terminator',13);
fprintf(s,msg);

function output = readserial()
global s;
set(s,'Terminator',10);
output = fscanf(s, '%c');
pause(.1);

function callerror(handles)
global intflag;
intflag = true;

set(handles.pushbutton2, 'Enable', 'off');
set(handles.pushbutton3, 'Enable', 'off');
set(handles.pushbutton4, 'Enable', 'off');
set(handles.pushbutton5, 'Enable', 'off');
set(handles.pushbutton6, 'Enable', 'off');
set(handles.slider1, 'Enable', 'off');
set(handles.text2, 'String', 'DISCONNECTED');
set(handles.popupmenu4, 'Enable', 'on');
set(handles.popupmenu5, 'Enable', 'off');
set(handles.popupmenu6, 'Enable', 'off');

h=errordlg('Connection error. Reconnect or try a different COM port.');
set(h, 'WindowStyle', 'modal');
uiwait(h);


function output = sendSerialCommand(handles, command, value)
global s;
global intflag;
global cenx;
global ceny;

output = '';

if (intflag)
    return;
end

switch command
    case 'con.set'
        index_selected = get(handles.popupmenu4,'Value');
        list = get(handles.popupmenu4,'String');
        
        s = serial(list{index_selected});
        set(s,'BaudRate',9600);
        set(s,'Terminator',13);
        set(s,'Timeout',3);
        set(s,'ErrorFcn',{@callerror,handles});
        
        try
            fopen(s);
        catch 
            h=errordlg('COM port unavailable.');
            set(h, 'WindowStyle', 'modal');
            uiwait(h);
        end
        
        writeserial('?R');
        output = readserial();
    case 'speed.ask'
        writeserial('?V');
        output = readserial();
    case 'pos.ask.x'
        writeserial('?X');
        output = readserial();
    case 'pos.ask.y'
        writeserial('?Y');
        output = readserial();
    case 'origin.ask'
        writeserial('?H');
        output = readserial();
        
    case 'speed.set'
        writeserial(strcat('V', num2str(value)));
        output = readserial();
    case 'pos.set.x'
        if (value < 0 || value > 50)
            h=errordlg('X value must be between 0 and 50 mm');
            set(h, 'WindowStyle', 'modal');
            uiwait(h);
            
            return;
        end
        
        out = sendSerialCommand(handles, 'pos.ask.x', '');
        C = strsplit(out,'+');
        x = 0.0025*str2num(C{2});
        
        value = round((value - x) / 0.0025);
        if (value > 0)
            writeserial(strcat('X+', num2str(value,'%d')));
        else
            writeserial(strcat('X', num2str(value,'%d')));
        end
        output = readserial();
    case 'pos.set.y'
        if (value < 0 || value > 50)
            h=errordlg('Y value must be between 0 and 50 mm');
            set(h, 'WindowStyle', 'modal');
            uiwait(h);
            
            return;
        end
        
        out = sendSerialCommand(handles, 'pos.ask.y', '');
        C = strsplit(out,'+');
        y = 0.0025*str2num(C{2});
        
        value = round((value - y) / 0.0025);
        if (value > 0)
            writeserial(strcat('Y+', num2str(value,'%d')));
        else
            writeserial(strcat('Y', num2str(value,'%d')));
        end
        output = readserial();
    case 'origin.set.x'
        writeserial('HX0');
        output = readserial();
    case 'origin.set.y'
        writeserial('HY0');
        output = readserial();
    case 'center.set'
        sendSerialCommand(handles, 'pos.set.x', cenx);
        sendSerialCommand(handles, 'pos.set.y', ceny);
end

function [start_index, steps, dwell_time, total_time] = setSource(handles)
list           = get(handles.popupmenu2, 'String');
val            = get(handles.popupmenu2, 'Value');
src_name       = list{val};

[~,col]        = find(cellfun('length',regexp(handles.txt,src_name)) == 1);
dwell_time     = str2double(get(handles.edit3, 'String'));

start_index    = 1;
stop_index     = find(cellfun('length',regexp({handles.txt{:,col}},'stop')) == 1);
steps          = (stop_index - 1 - start_index)/2;

total_time     = dwell_time * steps;

set(handles.text15, 'String', secs2hms(total_time));

function runSourceSequence(hObject, handles)
global cenx;
global ceny;

[start_index, steps, dwell_time, total_time] = setSource(handles);

set(handles.popupmenu2, 'Enable', 'off');
set(handles.edit3, 'Enable', 'off');
set(handles.popupmenu5, 'Enable', 'off');
set(handles.popupmenu6, 'Enable', 'off');

list = get(handles.popupmenu5, 'String');
val  = get(handles.popupmenu5, 'Value');
divisions = str2num(list{val});

list       = get(handles.popupmenu2, 'String');
val        = get(handles.popupmenu2, 'Value');
src_name   = list{val};

log = fopen('runs.log','w');
fprintf(log, '%s : %s, %d s - %d divisions\n', datestr(datetime('now','Format','d-MMM-y HH:mm:ss')), src_name, dwell_time, divisions);
fclose(log);

dwell_time = dwell_time / divisions;

for k=1:divisions
    for i=0:steps-1
        x = handles.num(start_index + i*2);
        y = handles.num(start_index + 1 + i*2);

        sendSerialCommand(handles, 'pos.set.x', cenx + x);
        sendSerialCommand(handles, 'pos.set.y', ceny + y);

        askXY(handles);

        for j=0:floor(dwell_time)
            if (dwell_time - j > 1)
                pause(1)
                set(handles.text15, 'String', secs2hms(total_time - (k-1)/divisions*total_time - dwell_time*i - j - 1));
            else
                pause(dwell_time - j)
                set(handles.text15, 'String', secs2hms(total_time - (k-1)/divisions*total_time - dwell_time*(i+1)));
            end

            handles = guidata(hObject);
            if (handles.interrupt == true)
                break;
            end
        end

        if (handles.interrupt == true)
            break;
        end
    end
end

set(handles.text15, 'String', secs2hms(total_time));

set(handles.popupmenu2, 'Enable', 'on');
set(handles.edit3, 'Enable', 'on');
set(handles.popupmenu5, 'Enable', 'on');
set(handles.popupmenu6, 'Enable', 'on');

pushbutton3_Callback(hObject, hObject, handles);

handles.interrupt = false;
guidata(hObject,handles);

askXY(handles);

function askXY(handles)
out = sendSerialCommand(handles, 'pos.ask.x', '');
C = strsplit(out,'+');
set(handles.text3, 'String', num2str(0.0025*str2num(C{2}), '%1.2f'));

out = sendSerialCommand(handles, 'pos.ask.y', '');
C = strsplit(out,'+');
set(handles.text5, 'String', num2str(0.0025*str2num(C{2}), '%1.2f'));

%%% CALLBACKS %%%
function pushbutton1_Callback(hObject, ~, handles)
global intflag;

index_selected = get(handles.popupmenu4,'Value');
list = get(handles.popupmenu4,'String');
if (length(instrfind('Port',list{index_selected})) > 0)
    fclose(instrfind('Port',list{index_selected}));
end

sendSerialCommand(handles, 'con.set', '');
sendSerialCommand(handles, 'speed.set', round(255*get(hObject,'Value')));

pushbutton3_Callback(hObject, hObject, handles);

set(handles.pushbutton2, 'Enable', 'on');
set(handles.pushbutton3, 'Enable', 'on');
set(handles.pushbutton4, 'Enable', 'on');
set(handles.pushbutton5, 'Enable', 'on');
set(handles.pushbutton6, 'Enable', 'on');
set(handles.slider1, 'Enable', 'on');
set(handles.text2, 'String', 'CONNECTED');
set(handles.popupmenu4, 'Enable', 'off');
set(handles.popupmenu5, 'Enable', 'on');
set(handles.popupmenu6, 'Enable', 'on');

intflag = false;

function pushbutton2_Callback(~, ~, handles)
sendSerialCommand(handles, 'pos.set.x', str2num(get(handles.edit1,'String')));
sendSerialCommand(handles, 'pos.set.y', str2num(get(handles.edit2,'String')));
askXY(handles);

function slider1_Callback(hObject, ~, handles)
set(handles.text10, 'String', strcat(num2str(100*get(hObject,'Value'),3), '%'));
sendSerialCommand(handles, 'speed.set', round(255*get(hObject,'Value')));

function pushbutton3_Callback(~, ~, handles)
sendSerialCommand(handles, 'origin.set.x', '');
sendSerialCommand(handles, 'origin.set.y', '');
askXY(handles);

function pushbutton4_Callback(~, ~, handles)
sendSerialCommand(handles, 'center.set', '');
askXY(handles);

function pushbutton5_Callback(hObject, ~, handles)
runSourceSequence(hObject, handles);

function pushbutton6_Callback(hObject, ~, handles)
handles.interrupt = true;
guidata(hObject,handles);
askXY(handles);

function popupmenu2_Callback(~, ~, handles)
setSource(handles);

function edit3_Callback(~, ~, handles)
setSource(handles);
popupmenu5_Callback(0, 0, handles);

function popupmenu5_Callback(~, ~, handles)
list = get(handles.popupmenu5, 'String');
val  = get(handles.popupmenu5, 'Value');
divisions      = str2num(list{val});
dwell_time     = str2double(get(handles.edit3, 'String'));

if (dwell_time/divisions <= 100)
    h=errordlg('Dwell times below 100s are strongly discouraged due to induced dose uncertainties.');
    set(h, 'WindowStyle', 'modal');
    uiwait(h);
    
    return;
end

function popupmenu6_Callback(hObject, eventdata, handles)
global cenx;
global ceny;

list = get(handles.popupmenu6, 'String');
val  = get(handles.popupmenu6, 'Value');

cenx = 23;
ceny = 21.7;

switch list{val}
    case 'default'
        cenx = cenx + 0;
        ceny = ceny + 0;
    case '1'
        cenx = cenx + 0.1754;
        ceny = ceny + 0.1519;
    case '2'
        cenx = cenx + 1.228;
        ceny = ceny + 2.279;
    case '3'
        cenx = cenx + 2.2806;
        ceny = ceny + 4.4061;
    case '4'
        cenx = cenx - 1.5789;
        ceny = ceny + 3.7983;
    case '5'
        cenx = cenx - 2.6315;
        ceny = ceny - 1.3674;
    case '6'
        cenx = cenx + 0.5263;
        ceny = ceny - 2.8868;
    case '7'
        cenx = cenx + 4.035;
        ceny = ceny - 1.6713;
    case '8'
        cenx = cenx + 2.9824;
        ceny = ceny + 0.4558;
    case '9'
        cenx = cenx - 1.228;
        ceny = ceny + 0.7579;
end

sendSerialCommand(handles, 'center.set', '');
askXY(handles);

%%% CREATOR FUNCTIONS %%%
function interface_OpeningFcn(hObject, ~, handles, varargin)
handles.output              = hObject;

handles.interrupt           = false;
[handles.num,handles.txt]   = xlsread('position_data.xlsx');
set(handles.popupmenu2, 'String', handles.txt(1,:));
guidata(hObject,handles);

setSource(handles);

function varargout = interface_OutputFcn(~, ~, handles) 
varargout{1} = handles.output;

function edit1_CreateFcn(hObject, ~, ~)

if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end

function edit2_CreateFcn(hObject, ~, ~)

if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end

function edit3_CreateFcn(hObject, ~, ~)

if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end

function edit4_CreateFcn(hObject, ~, ~)

if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end

function popupmenu2_CreateFcn(hObject, ~, ~)

if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end

function popupmenu3_CreateFcn(hObject, ~, ~)

if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end

function popupmenu4_CreateFcn(hObject, ~, ~)

if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end

function slider1_CreateFcn(hObject, ~, ~)

if isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor',[.9 .9 .9]);
end

function popupmenu5_CreateFcn(hObject, eventdata, handles)
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end

function popupmenu6_CreateFcn(hObject, eventdata, handles)
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end
